@@ -2,7 +2,6 @@
 
 ;; Author: Torsten Hilbrich <dictionary@myrkr.in-berlin.de>
 ;; Keywords: interface, dictionary
-;; $Id: dictionary.el 338 2004-10-02 06:04:54Z torsten $
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -41,6 +40,15 @@
     (defmacro defcustom (var value doc &rest ignored)
       (list 'defvar var value doc))))
 
+(defun dictionary-set-server-var (name value)
+  (if (and (boundp 'dictionary-connection)
+	   dictionary-connection
+	   (eq (connection-status dictionary-connection) 'up)
+	   (y-or-n-p
+	    (concat "Close existing connection to " dictionary-server "? ")))
+      (connection-close dictionary-connection))
+  (set-default name value))
+
 (defgroup dictionary nil
   "Client for accessing the dictd server based dictionaries"
   :group 'hypermedia)
@@ -53,6 +61,7 @@
   "dict.org"
   "This server is contacted for searching the dictionary"
   :group 'dictionary
+  :set 'dictionary-set-server-var
   :type 'string)
 
 (defcustom dictionary-port
@@ -60,6 +69,7 @@
   "The port of the dictionary server.
  This port is propably always 2628 so there should be no need to modify it."
   :group 'dictionary
+  :set 'dictionary-set-server-var
   :type 'number)
 
 (defcustom dictionary-identification
@@ -135,18 +145,21 @@ by the choice value:
   nil
   "Connects via a HTTP proxy using the CONNECT command when not nil."
   :group 'dictionary-proxy
+  :set 'dictionary-set-server-var
   :type 'boolean)
 
 (defcustom dictionary-proxy-server
   "proxy"
   "The name of the HTTP proxy to use when dictionary-use-http-proxy is set."
   :group 'dictionary-proxy
+  :set 'dictionary-set-server-var
   :type 'string)
 
 (defcustom dictionary-proxy-port
   3128
   "The port of the proxy server, used only when dictionary-use-http-proxy is set."
   :group 'dictionary-proxy
+  :set 'dictionary-set-server-var
   :type 'number)
 
 (defcustom dictionary-use-single-buffer
@@ -282,7 +295,7 @@ by the choice value:
   (condition-case nil
       (x-display-color-p)
     (error nil))
-  "Stores the point position while buffer display.")
+  "Determines if the Emacs has support to display color")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Basic function providing startup actions
@@ -330,7 +343,8 @@ by the choice value:
   (make-local-variable 'dictionary-default-dictionary)
   (make-local-variable 'dictionary-default-strategy)
   
-  (make-local-hook 'kill-buffer-hook)
+  (if (featurep 'xemacs)
+      (make-local-hook 'kill-buffer-hook))
   (add-hook 'kill-buffer-hook 'dictionary-close t t)
   (run-hooks 'dictionary-mode-hook))
 
@@ -412,6 +426,8 @@ by the choice value:
 		    (connection-open dictionary-proxy-server 
 				     dictionary-proxy-port)
 		  (connection-open dictionary-server dictionary-port)))
+	  (process-kill-without-query
+	   (connection-process dictionary-connection))
 	  
 	  (when dictionary-use-http-proxy
 	    (message "Proxy CONNECT to %s:%d" 
@@ -635,7 +651,9 @@ This function knows about the special meaning of quotes (\")"
   "The workhorse for doing the search"
   
   (message "Searching for %s in %s" word dictionary)
-  (dictionary-send-command (concat "define " dictionary " \""
+  (dictionary-send-command (concat "define "
+				   (dictionary-encode-charset dictionary "")
+				   " \""
 				   (dictionary-encode-charset word dictionary)
 				   "\""))
   
@@ -746,18 +764,17 @@ This function knows about the special meaning of quotes (\")"
       (goto-char start)
       (while (< (point) (point-max))
 	(if (search-forward-regexp regexp nil t)
-	    (let ((match-start (match-beginning 1))
+	    (let ((match-start (match-beginning 2))
 		  (match-end (match-end 2)))
 	      (if dictionary-color-support
-		  (progn
-		    (replace-match "\\2")
-		    ;; Compensate for the replacement
-		    (let ((brace-match-length (- (match-end 1)
-						 (match-beginning 1))))
-		      (setq match-start (- (match-beginning 2)
-					   brace-match-length))
-		      (setq match-end (- (match-end 2)
-					 brace-match-length)))))
+		  ;; Compensate for the replacement
+		  (let ((brace-match-length (- (match-end 1)
+					       (match-beginning 1))))
+		    (setq match-start (- (match-beginning 2)
+					 brace-match-length))
+		    (setq match-end (- (match-end 2)
+				       brace-match-length))
+		    (replace-match "\\2")))
 	      (dictionary-mark-reference match-start match-end
 					 'dictionary-new-search
 					 word dictionary))
@@ -832,7 +849,8 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
 	 (description (cadr list))
 	 (translated (dictionary-decode-charset description dictionary)))
     (if dictionary
-	(progn
+	(if (equal dictionary "--exit--")
+	    (insert "(end of default search list)\n")
 	  (link-insert-link (concat dictionary ": " translated)
 			    'dictionary-reference-face
 			    'dictionary-set-dictionary 
@@ -859,7 +877,8 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
 		(equal dictionary "!"))
       (dictionary-store-positions)
       (message "Requesting more information on %s" dictionary)
-      (dictionary-send-command (concat "show info " dictionary))
+      (dictionary-send-command
+       (concat "show info " (dictionary-encode-charset dictionary "")))
       (let ((reply (dictionary-read-reply-and-split)))
 	(message nil)
 	(if (dictionary-check-reply reply 550)
@@ -948,8 +967,9 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
   (message "Lookup matching words for %s in %s using %s"
 	   word dictionary strategy)
   (dictionary-send-command 
-   (concat "match " dictionary " "
-	   strategy " \"" (dictionary-encode-charset word "") "\""))
+   (concat "match " (dictionary-encode-charset dictionary "") " "
+	   (dictionary-encode-charset strategy "") " \""
+	   (dictionary-encode-charset word "") "\""))
   (let ((reply (dictionary-read-reply-and-split)))
     (message nil)
     (if (dictionary-check-reply reply 550)
@@ -1038,9 +1058,16 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
   "Search the `word' in `dictionary' if given or in all if nil.  
 It presents the word at point as default input and allows editing it."
   (interactive
-   (list (read-string "Search word: " (current-word))
+   (list (let ((default (current-word t)))
+           (read-string (if default
+                            (format "Search word (%s): " default)
+                          "Search word: ")
+                        nil nil default))
 	 (if current-prefix-arg
-	     (read-string "Dictionary: " dictionary-default-dictionary)
+	     (read-string (if dictionary-default-dictionary
+			      (format "Dictionary (%s): " dictionary-default-dictionary)
+			    "Dictionary: ")
+			  nil nil dictionary-default-dictionary)
 	   dictionary-default-dictionary)))
   
   ;; if called by pressing the button
